"""Module `dictIO.cppDict`.

_Note_: Deprecated since 0.4.0.
Module `dictIO.cppDict` and class `CppDict` will be removed in 0.5.0.
Use module `dictIO.dict` and class `SDict` instead.
"""

from __future__ import annotations

import warnings
from typing import (
    TYPE_CHECKING,
    Any,
    overload,
)

from dictIO.dict import SDict

if TYPE_CHECKING:
    import os
    from _collections_abc import Iterable, Mapping


__ALL__ = [
    "CppDict",
]


class CppDict(SDict[str, Any]):
    """Data structure for C++ dictionaries.

    This class is deprecated and will be removed in 0.5.0.
    Use SDict[K, V] instead.
    """

    @overload
    def __init__(
        self,
        **kwargs: Any,  # noqa: ANN401
    ) -> None:
        pass

    @overload
    def __init__(
        self,
        arg: Mapping[str, Any],
        **kwargs: Any,  # noqa: ANN401
    ) -> None:
        pass

    @overload
    def __init__(
        self,
        arg: Iterable[tuple[str, Any]],
        **kwargs: Any,  # noqa: ANN401
    ) -> None:
        pass

    @overload
    def __init__(
        self,
        arg: str | os.PathLike[str],
        **kwargs: Any,  # noqa: ANN401
    ) -> None:
        pass

    def __init__(
        self,
        arg: Mapping[str, Any] | Iterable[tuple[str, Any]] | str | os.PathLike[str] | None = None,
        **kwargs: Any,
    ) -> None:
        warnings.warn(
            "`CppDict` is deprecated. Use `SDict[K, V]` instead.",
            DeprecationWarning,
            stacklevel=1,
        )
        super().__init__(arg, **kwargs)  # type: ignore[arg-type, reportArgumentType, reportCallIssue]
        return



import logging
import os
from pathlib import Path
from shutil import rmtree

import pytest


@pytest.fixture(scope="package", autouse=True)
def chdir() -> None:
    """
    Fixture that changes the current working directory to the 'test_working_directory' folder.
    This fixture is automatically used for the entire package.
    """
    os.chdir(Path(__file__).parent.absolute() / "test_working_directory")


@pytest.fixture(scope="package", autouse=True)
def test_dir() -> Path:
    """
    Fixture that returns the absolute path of the directory containing the current file.
    This fixture is automatically used for the entire package.
    """
    return Path(__file__).parent.absolute()


output_dirs: list[str] = []
output_files: list[str] = [
    "parsed.*",
]


@pytest.fixture(autouse=True)
def default_setup_and_teardown():
    """
    Fixture that performs setup and teardown actions before and after each test function.
    It removes the output directories and files specified in 'output_dirs' and 'output_files' lists.
    """
    _remove_output_dirs_and_files()
    yield
    _remove_output_dirs_and_files()


def _remove_output_dirs_and_files() -> None:
    """
    Helper function that removes the output directories and files specified in 'output_dirs' and 'output_files' lists.
    """
    for folder in output_dirs:
        rmtree(folder, ignore_errors=True)
    for pattern in output_files:
        for file in Path.cwd().glob(pattern):
            _file = Path(file)
            _file.unlink(missing_ok=True)


@pytest.fixture(autouse=True)
def setup_logging(caplog: pytest.LogCaptureFixture) -> None:
    """
    Fixture that sets up logging for each test function.
    It sets the log level to 'INFO' and clears the log capture.
    """
    caplog.set_level("INFO")
    caplog.clear()


@pytest.fixture(autouse=True)
def logger() -> logging.Logger:
    """Fixture that returns the logger object."""
    return logging.getLogger()



#!/usr/bin/env python
"""dictParser command line interface."""

import argparse
import logging
import re
from collections.abc import MutableSequence
from pathlib import Path
from typing import Any, cast

from dictIO import DictParser
from dictIO.types import K
from dictIO.utils.logging import configure_logging

logger = logging.getLogger(__name__)


def _argparser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="dictParser",
        usage="%(prog)s dict [options [args]]",
        epilog="_________________dictParser___________________",
        prefix_chars="-",
        add_help=True,
        description=(
            "Reads a dict file, merges sub-dicts referenced through #include directives,\n"
            "evaluates variables and expressions,\n"
            "and finally saves the parsed dict with prefix 'parsed', i.e. parsed.<DICTNAME>."
            "The format of the output file will by default be dictIO native file format, but can optionally be\n"
            "changed to foam, xml or json format."
        ),
    )

    _ = parser.add_argument(
        "dict",
        metavar="dict",
        type=str,
        help="name of dict file to be parsed.",
    )

    _ = parser.add_argument(
        "-I",
        "--ignore-includes",
        action="store_true",
        help=("ignore include directives (e.g. #include './SUBDICT').\nThis suppresses merging of sub-dicts. "),
        default=False,
        required=False,
    )

    _ = parser.add_argument(
        "--mode",
        help=(
            "'a' -- append to output file if a dict with the same name already exists; \n"
            "'w' -- overwrite output file (default)"
        ),
        choices=["a", "w"],
        default="w",
        required=False,
        type=str,
    )

    _ = parser.add_argument(
        "--order",
        action="store_true",
        help="sort the parsed dict.",
        default=False,
        required=False,
    )

    _ = parser.add_argument(
        "-C",
        "--ignore-comments",
        action="store_true",
        help="supress writing comments into the output file. The header is excepted and will always be written.",
        default=False,
        required=False,
    )

    _ = parser.add_argument(
        "--scope",
        action="store",
        help=(
            "optionally specify a scope the dict will be reduced to after parsing.\n"
            "'scope' can be EMPTY, a 'STRING' or a list of strings \"['STRING1', 'STRING2']\""
        ),
        default=None,
        required=False,
    )

    _ = parser.add_argument(
        "-o",
        "--output",
        action="store",
        type=str,
        help="specify format of the output file.",
        choices=["cpp", "foam", "xml", "json"],
        default="cpp",
        required=False,
    )

    console_verbosity = parser.add_mutually_exclusive_group(required=False)

    _ = console_verbosity.add_argument(
        "-q",
        "--quiet",
        action="store_true",
        help=("console output will be quiet."),
        default=False,
    )

    _ = console_verbosity.add_argument(
        "-v",
        "--verbose",
        action="store_true",
        help=("console output will be verbose."),
        default=False,
    )

    _ = parser.add_argument(
        "--log",
        action="store",
        type=str,
        help="name of log file. If specified, this will activate logging to file.",
        default=None,
        required=False,
    )

    _ = parser.add_argument(
        "--log-level",
        action="store",
        type=str,
        help="log level applied to logging to file.",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        default="WARNING",
        required=False,
    )

    return parser


def main() -> None:
    """Entry point for console script as configured in pyproject.toml.

    Runs the command line interface and parses arguments and options entered on the console.
    """
    parser = _argparser()
    args = parser.parse_args()

    # Configure Logging
    # ..to console
    log_level_console: str = "WARNING"
    if any([args.quiet, args.verbose]):
        log_level_console = "ERROR" if args.quiet else log_level_console
        log_level_console = "INFO" if args.verbose else log_level_console
    # ..to file
    log_file: Path | None = Path(args.log) if args.log else None
    log_level_file: str = args.log_level
    configure_logging(log_level_console, log_file, log_level_file)

    source_file = Path(args.dict)
    includes: bool = not args.ignore_includes
    mode: str = args.mode
    order: bool = args.order
    comments: bool = not args.ignore_comments
    # Validate scope: It needs to be a list of strings
    scope: MutableSequence[Any] | None = _validate_scope(args.scope)
    output: str | None = args.output

    # Check whether source file exists
    if not source_file.exists():
        logger.error(f"dictParser.py: File {source_file} not found.")
        return

    logger.info(
        f"Start dictParser.py with following arguments:\n"
        f"\t source_file: \t\t{source_file}\n"
        f"\t includes: \t\t\t{includes}\n"
        f"\t order: \t\t\t\t{order}\n"
        f"\t comments: \t\t\t{comments}\n"
        f"\t scope: \t\t\t\t{scope}\n"
        f"\t output: \t\t\t{output}"
    )

    # Invoke API
    if DictParser.parse(
        source_file=source_file,
        includes=includes,
        mode=mode,
        order=order,
        comments=comments,
        scope=scope,
        output=output,
    ):
        logger.info("dictParser.py finished successfully.\n")
    else:
        logger.error("dictParser.py finished with errors.\n")


def _validate_scope(
    scope: MutableSequence[K] | K | None,
) -> list[K] | None:
    # sourcery skip: replace-interpolation-with-fstring
    validated_scope: list[K] | None = None
    if isinstance(scope, MutableSequence):  # List
        validated_scope = list(scope)  # no conversion needed
    elif isinstance(scope, str):  # string
        if re.match(r"^\s*\[", scope):  # ..maybe a string that LOOKS like a list?
            try:  # Then try to convert that string to a list
                from dictIO import Parser  # noqa: PLC0415

                parser = Parser()
                _scope: str = scope.strip(" []")
                validated_scope = cast("list[K]", [key.strip() for key in _scope.split(",")])
                parser.parse_values(validated_scope)
            except Exception:
                logger.exception(f"setOptions: misspelled scope: {scope}")
        else:  # string is just a single value.
            # Store it not as string but as a (one-element) list
            validated_scope = [cast("K", scope)]
    else:  # 'scope' is neither a list nor a string -> set validated_scope to None
        validated_scope = None
    return validated_scope


if __name__ == "__main__":
    main()



import sys
from pathlib import Path

import pytest

from dictIO.utils.path import highest_common_root_folder, relative_path

WindowsOnly: pytest.MarkDecorator = pytest.mark.skipif(not sys.platform.startswith("win"), reason="windows only test")


def test_highest_common_root_folder() -> None:
    # Prepare
    file_01: Path = Path(r"C:/A0/A1/A2/file_01.abc")
    file_02: Path = Path(r"C:/A0/A1/A2/file_02.abc")
    file_03: Path = Path(r"C:/A0/A1/B2/file_03.abc")
    file_04: Path = Path(r"C:/A0/B1/B2/file_04.abc")
    file_05: Path = Path(r"C:/B0/B1/B2/file_05.abc")
    file_06: Path = Path(r"C:/A0/A1/file_06.abc")
    file_07: Path = Path(r"C:/A0/B1/file_07.abc")
    file_08: Path = Path(r"C:/B0/B1/file_08.abc")
    file_09: Path = Path(r"C:/A0/file_09.abc")
    file_10: Path = Path(r"C:/B0/file_10.abc")
    file_11: Path = Path(r"C:/file_11.abc")
    file_12: Path = Path(r"D:/file_12.abc")
    folder_01: Path = Path(r"C:/A0/A1/A2/")
    folder_02: Path = Path(r"C:/A0/A1/A2/")
    folder_03: Path = Path(r"C:/A0/A1/B2/")
    folder_04: Path = Path(r"C:/A0/B1/B2/")
    folder_05: Path = Path(r"C:/B0/B1/B2/")
    folder_06: Path = Path(r"C:/A0/A1/")
    folder_07: Path = Path(r"C:/A0/B1/")
    folder_08: Path = Path(r"C:/B0/B1/")
    folder_09: Path = Path(r"C:/A0/")
    folder_10: Path = Path(r"C:/B0/")
    folder_11: Path = Path(r"C:/")
    folder_12: Path = Path(r"D:/")
    # Execute and Assert
    assert highest_common_root_folder([file_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04]) == folder_09.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04, file_05]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([file_01, file_06]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([file_01, file_07]) == folder_09.resolve().absolute()
    assert highest_common_root_folder([file_01, file_08]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([file_01, file_09]) == folder_09.resolve().absolute()
    assert highest_common_root_folder([file_01, file_10]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([file_04, file_07]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([file_05, file_08]) == folder_08.resolve().absolute()
    assert highest_common_root_folder([file_08, file_10]) == folder_10.resolve().absolute()
    assert highest_common_root_folder([file_11]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([file_12]) == folder_12.resolve().absolute()
    assert highest_common_root_folder([folder_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03, folder_04]) == folder_09.resolve().absolute()
    assert (
        highest_common_root_folder([folder_01, folder_02, folder_03, folder_04, folder_05])
        == folder_11.resolve().absolute()
    )
    assert highest_common_root_folder([folder_01, folder_06]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_07]) == folder_09.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_08]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_09]) == folder_09.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_10]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([folder_04, folder_07]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([folder_05, folder_08]) == folder_08.resolve().absolute()
    assert highest_common_root_folder([folder_08, folder_10]) == folder_10.resolve().absolute()
    assert highest_common_root_folder([folder_11]) == folder_11.resolve().absolute()
    assert highest_common_root_folder([folder_12]) == folder_12.resolve().absolute()


@WindowsOnly
def test_highest_common_root_folder_raises_value_error() -> None:
    # Prepare
    file_11: Path = Path(r"C:/file_11.abc")
    file_12: Path = Path(r"D:/file_12.abc")
    folder_11: Path = Path(r"C:/")
    folder_12: Path = Path(r"D:/")
    # Execute and Assert
    with pytest.raises(ValueError):
        _ = highest_common_root_folder([])
    with pytest.raises(ValueError):
        _ = highest_common_root_folder([file_11, file_12])
    with pytest.raises(ValueError):
        _ = highest_common_root_folder([folder_11, folder_12])


def test_highest_common_root_folder_with_multiple_occurences_of_similar_parts_1() -> None:
    # Prepare
    file_01: Path = Path(r"C:/A0/A1/A0/A1/A0/A1/file_01.abc")
    file_02: Path = Path(r"C:/A0/A1/A0/A1/A0/file_02.abc")
    file_03: Path = Path(r"C:/A0/A1/A0/A1/file_03.abc")
    file_04: Path = Path(r"C:/A0/A1/A0/file_04.abc")
    file_05: Path = Path(r"C:/A0/A1/file_05.abc")
    file_06: Path = Path(r"C:/A0/file_06.abc")
    file_07: Path = Path(r"C:/file_07.abc")

    folder_01: Path = Path(r"C:/A0/A1/A0/A1/A0/A1/")
    folder_02: Path = Path(r"C:/A0/A1/A0/A1/A0/")
    folder_03: Path = Path(r"C:/A0/A1/A0/A1/")
    folder_04: Path = Path(r"C:/A0/A1/A0/")
    folder_05: Path = Path(r"C:/A0/A1/")
    folder_06: Path = Path(r"C:/A0/")
    folder_07: Path = Path(r"C:/")
    # Execute and Assert
    assert highest_common_root_folder([file_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02]) == folder_02.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03]) == folder_03.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04]) == folder_04.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04, file_05]) == folder_05.resolve().absolute()
    assert (
        highest_common_root_folder([file_01, file_02, file_03, file_04, file_05, file_06])
        == folder_06.resolve().absolute()
    )
    assert (
        highest_common_root_folder([file_01, file_02, file_03, file_04, file_05, file_06, file_07])
        == folder_07.resolve().absolute()
    )

    assert highest_common_root_folder([folder_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02]) == folder_02.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03]) == folder_03.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03, folder_04]) == folder_04.resolve().absolute()
    assert (
        highest_common_root_folder([folder_01, folder_02, folder_03, folder_04, folder_05])
        == folder_05.resolve().absolute()
    )
    assert (
        highest_common_root_folder([folder_01, folder_02, folder_03, folder_04, folder_05, folder_06])
        == folder_06.resolve().absolute()
    )
    assert (
        highest_common_root_folder(
            [
                folder_01,
                folder_02,
                folder_03,
                folder_04,
                folder_05,
                folder_06,
                folder_07,
            ]
        )
        == folder_07.resolve().absolute()
    )


def test_highest_common_root_folder_with_multiple_occurences_of_similar_parts_2() -> None:
    # Prepare
    file_01: Path = Path(r"C:/A0/A0/A0/A0/A0/A0/file_01.abc")
    file_02: Path = Path(r"C:/A0/A0/A0/A0/A0/file_02.abc")
    file_03: Path = Path(r"C:/A0/A0/A0/A0/file_03.abc")
    file_04: Path = Path(r"C:/A0/A0/A0/file_04.abc")
    file_05: Path = Path(r"C:/A0/A0/file_05.abc")
    file_06: Path = Path(r"C:/A0/file_06.abc")
    file_07: Path = Path(r"C:/file_07.abc")

    folder_01: Path = Path(r"C:/A0/A0/A0/A0/A0/A0/")
    folder_02: Path = Path(r"C:/A0/A0/A0/A0/A0/")
    folder_03: Path = Path(r"C:/A0/A0/A0/A0/")
    folder_04: Path = Path(r"C:/A0/A0/A0/")
    folder_05: Path = Path(r"C:/A0/A0/")
    folder_06: Path = Path(r"C:/A0/")
    folder_07: Path = Path(r"C:/")
    # Execute and Assert
    assert highest_common_root_folder([file_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02]) == folder_02.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03]) == folder_03.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04]) == folder_04.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04, file_05]) == folder_05.resolve().absolute()
    assert (
        highest_common_root_folder([file_01, file_02, file_03, file_04, file_05, file_06])
        == folder_06.resolve().absolute()
    )
    assert (
        highest_common_root_folder([file_01, file_02, file_03, file_04, file_05, file_06, file_07])
        == folder_07.resolve().absolute()
    )

    assert highest_common_root_folder([folder_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02]) == folder_02.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03]) == folder_03.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03, folder_04]) == folder_04.resolve().absolute()
    assert (
        highest_common_root_folder([folder_01, folder_02, folder_03, folder_04, folder_05])
        == folder_05.resolve().absolute()
    )
    assert (
        highest_common_root_folder([folder_01, folder_02, folder_03, folder_04, folder_05, folder_06])
        == folder_06.resolve().absolute()
    )
    assert (
        highest_common_root_folder(
            [
                folder_01,
                folder_02,
                folder_03,
                folder_04,
                folder_05,
                folder_06,
                folder_07,
            ]
        )
        == folder_07.resolve().absolute()
    )


def test_highest_common_root_folder_with_multiple_occurences_of_similar_parts_3() -> None:
    # Prepare
    file_01: Path = Path(r"C:/A0/A0/A0/A0/A0/A1/file_01.abc")
    file_02: Path = Path(r"C:/A0/A1/A0/A0/A0/A0/file_02.abc")
    file_03: Path = Path(r"C:/A0/A0/A0/A0/A1/file_03.abc")
    file_04: Path = Path(r"C:/A1/A0/A0/file_04.abc")
    file_05: Path = Path(r"C:/A0/A0/file_05.abc")
    file_06: Path = Path(r"C:/A0/file_06.abc")
    file_07: Path = Path(r"C:/file_07.abc")

    folder_01: Path = Path(r"C:/A0/A0/A0/A0/A0/A1/")
    folder_02: Path = Path(r"C:/A0/A1/A0/A0/A0/A0/")
    folder_03: Path = Path(r"C:/A0/A0/A0/A0/A1/")
    folder_04: Path = Path(r"C:/A1/A0/A0/")
    folder_05: Path = Path(r"C:/A0/A0/")
    folder_06: Path = Path(r"C:/A0/")
    folder_07: Path = Path(r"C:/")
    # Execute and Assert
    assert highest_common_root_folder([file_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([file_01, file_02, file_03, file_04]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([file_01, file_05]) == folder_05.resolve().absolute()
    assert highest_common_root_folder([file_01, file_06]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([file_01, file_07]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([file_02, file_03]) == folder_06.resolve().absolute()

    assert highest_common_root_folder([folder_01]) == folder_01.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_02, folder_03, folder_04]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_05]) == folder_05.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_06]) == folder_06.resolve().absolute()
    assert highest_common_root_folder([folder_01, folder_07]) == folder_07.resolve().absolute()
    assert highest_common_root_folder([folder_02, folder_03]) == folder_06.resolve().absolute()


def test_relative_path() -> None:
    # Prepare
    file_01: Path = Path(r"C:/A0/A1/A2/file_01.abc")
    file_06: Path = Path(r"C:/A0/A1/file_06.abc")
    folder_01: Path = Path(r"C:/A0/A1/A2/")
    folder_06: Path = Path(r"C:/A0/A1/")
    folder_07: Path = Path(r"C:/A0/B1/")
    folder_08: Path = Path(r"C:/B0/B1/")
    folder_09: Path = Path(r"C:/A0/")
    folder_10: Path = Path(r"C:/B0/")
    folder_11: Path = Path(r"C:/")

    relative_path_folder_06_folder_01 = Path(r"A2/")
    relative_path_folder_01_folder_06 = Path(r"../")

    relative_path_folder_09_folder_01 = Path(r"A1/A2/")
    relative_path_folder_01_folder_09 = Path(r"../../")

    relative_path_folder_11_folder_01 = Path(r"A0/A1/A2/")
    relative_path_folder_01_folder_11 = Path(r"../../../")

    relative_path_folder_09_folder_06 = Path(r"A1/")
    relative_path_folder_06_folder_09 = Path(r"../")

    relative_path_folder_11_folder_06 = Path(r"A0/A1/")
    relative_path_folder_06_folder_11 = Path(r"../../")

    relative_path_folder_11_folder_09 = Path(r"A0/")
    relative_path_folder_09_folder_11 = Path(r"../")

    relative_path_folder_07_folder_01 = Path(r"../A1/A2/")
    relative_path_folder_01_folder_07 = Path(r"../../B1/")

    relative_path_folder_08_folder_01 = Path(r"../../A0/A1/A2/")
    relative_path_folder_01_folder_08 = Path(r"../../../B0/B1/")

    relative_path_folder_10_folder_01 = Path(r"../A0/A1/A2/")
    relative_path_folder_01_folder_10 = Path(r"../../../B0/")

    relative_path_folder_06_file_01 = Path(r"A2/file_01.abc")
    relative_path_folder_01_file_06 = Path(r"../file_06.abc")

    relative_path_file_06_folder_01 = Path(r"../A2/")
    relative_path_file_01_folder_06 = Path(r"../../")

    relative_path_file_06_file_01 = Path(r"../A2/file_01.abc")
    relative_path_file_01_file_06 = Path(r"../../file_06.abc")

    # Execute and Assert
    assert relative_path(folder_06, folder_01) == relative_path_folder_06_folder_01
    assert relative_path(folder_01, folder_06) == relative_path_folder_01_folder_06

    assert relative_path(folder_09, folder_01) == relative_path_folder_09_folder_01
    assert relative_path(folder_01, folder_09) == relative_path_folder_01_folder_09

    assert relative_path(folder_11, folder_01) == relative_path_folder_11_folder_01
    assert relative_path(folder_01, folder_11) == relative_path_folder_01_folder_11

    assert relative_path(folder_09, folder_06) == relative_path_folder_09_folder_06
    assert relative_path(folder_06, folder_09) == relative_path_folder_06_folder_09

    assert relative_path(folder_11, folder_06) == relative_path_folder_11_folder_06
    assert relative_path(folder_06, folder_11) == relative_path_folder_06_folder_11

    assert relative_path(folder_11, folder_09) == relative_path_folder_11_folder_09
    assert relative_path(folder_09, folder_11) == relative_path_folder_09_folder_11

    assert relative_path(folder_07, folder_01) == relative_path_folder_07_folder_01
    assert relative_path(folder_01, folder_07) == relative_path_folder_01_folder_07

    assert relative_path(folder_08, folder_01) == relative_path_folder_08_folder_01
    assert relative_path(folder_01, folder_08) == relative_path_folder_01_folder_08

    assert relative_path(folder_10, folder_01) == relative_path_folder_10_folder_01
    assert relative_path(folder_01, folder_10) == relative_path_folder_01_folder_10

    assert relative_path(folder_06, file_01) == relative_path_folder_06_file_01
    assert relative_path(folder_01, file_06) == relative_path_folder_01_file_06

    assert relative_path(file_06, folder_01) == relative_path_file_06_folder_01
    assert relative_path(file_01, folder_06) == relative_path_file_01_folder_06

    assert relative_path(file_06, file_01) == relative_path_file_06_file_01
    assert relative_path(file_01, file_06) == relative_path_file_01_file_06


@WindowsOnly
def test_relative_path_raises_value_error() -> None:
    # Prepare
    file_11: Path = Path(r"C:/file_11.abc")
    file_12: Path = Path(r"D:/file_12.abc")
    folder_11: Path = Path(r"C:/")
    folder_12: Path = Path(r"D:/")

    # Execute and Assert
    with pytest.raises(ValueError):
        _ = relative_path(folder_11, folder_12)
    with pytest.raises(ValueError):
        _ = relative_path(folder_11, file_12)
    with pytest.raises(ValueError):
        _ = relative_path(file_11, folder_12)
    with pytest.raises(ValueError):
        _ = relative_path(file_11, file_12)
